@@ -2,7 +2,7 @@
 #
 # Author: Mike McKerns (mmckerns @caltech and @uqfoundation)
 # Copyright (c) 2008-2016 California Institute of Technology.
-# Copyright (c) 2016-2021 The Uncertainty Quantification Foundation.
+# Copyright (c) 2016-2022 The Uncertainty Quantification Foundation.
 # License: 3-clause BSD.  The full license text is available at:
 #  - https://github.com/uqfoundation/dill/blob/master/LICENSE
 
@@ -55,18 +55,15 @@ def test_diff():
     assert changed[1]
 
     if not IS_PYPY:
-        try:
-            import abc
-            # make sure the "_abc_invaldation_counter" doesn't make test fail
-            diff.memorise(abc.ABCMeta, force=True)
-            assert not diff.has_changed(abc)
-            abc.ABCMeta.zzz = 1
-            assert diff.has_changed(abc)
-            changed = diff.whats_changed(abc)
-            assert list(changed[0].keys()) == ["ABCMeta"]
-            assert not changed[1]
-        except ImportError:
-            pass
+        import abc
+        # make sure the "_abc_invaldation_counter" doesn't make test fail
+        diff.memorise(abc.ABCMeta, force=True)
+        assert not diff.has_changed(abc)
+        abc.ABCMeta.zzz = 1
+        assert diff.has_changed(abc)
+        changed = diff.whats_changed(abc)
+        assert list(changed[0].keys()) == ["ABCMeta"]
+        assert not changed[1]
 
     '''
     import Queue
